# This script contains most of the analyses described in Sections 5.1 - 5.4

# Change the following to the appropriate working directory
setwd("D:/informationrevelation/code/analysis/");

# Loading required R packages
require(data.table);
require(lfe);       # Used for regressions with clustered standard errors
require(stargazer); # Used for reporting regression results in table format
require(ggrepel);   # Used for plots in Fig 4 & 5

# Loading subsample network stats (from script 1g)
load(file=paste0("input/subsampleStats"));
subset = subsampleStats$eventTime==5;

# Table 1: Adjacency matrix --> produced in script 2b

# Table 2: Network properties
out = aggregate(which(subset), by=list(subsampleStats$mkt[subset]), function(rows) colMeans(subsampleStats[rows,c("ISl","speed","PI","ED_0.2","ED_0.1","ED_all"),with=F]))
Table2 = out$x[match(c("EBS","FXN0","DCFX","TFS0","DUBA","PFD0","WBRS","BBBB","PEP0"),out$Group.1),]
dimnames(Table2)[[1]] = c("EBS",paste0("DLR",1:8));
Table2[,"ISl"] = 100* Table2[,"ISl"];
capture.output(round(Table2,2), file = "output/Table2.txt")


# Table 3A: Network assortativity
Table3A = rbind(c(t.test(subsampleStats$assort_0.1[subset])$estimate,
                  t.test(subsampleStats$assort_0.1[subset])$stat),
                c(t.test(subsampleStats$assort_0.2[subset])$estimate,
                  t.test(subsampleStats$assort_0.2[subset])$stat))

dimnames(Table3A) = list(c("ED_0.1","ED_0.2"),c("assortCorr","t"))
capture.output(round(Table3A[2:1,],2), file = "output/Table3A.txt")

# Table 3B: Network stability --> produced in script 2b

# Table 4: Granger causality regressions with standardized coefficients
#          Based on tau-market observations averaged across dates (retaining the mkt and intraday dimensions)
meanStatsTime = subsampleStats[eventTime %in% 1:5 & mkt != "EBS",list(dPI     = mean(dPI,na.rm=T),     dPI.lag     = mean(dPI.lag,na.rm=T),
                                                                      dED_0.1 = mean(dED_0.1,na.rm=T), dED_0.1.lag = mean(dED_0.1.lag,na.rm=T),
                                                                      dED_0.2 = mean(dED_0.2,na.rm=T), dED_0.2.lag = mean(dED_0.2.lag,na.rm=T),
                                                                      dED_all = mean(dED_all,na.rm=T), dED_all.lag = mean(dED_all.lag,na.rm=T)),by=list(mkt,eventTime,time)]

regr_0.2     = felm(scale(dPI)     ~ scale(dPI.lag) + scale(dED_0.2.lag) - 1 | 0 | 0 | mkt + eventTime + time, data = meanStatsTime);
regr_0.2.rev = felm(scale(dED_0.2) ~ scale(dPI.lag) + scale(dED_0.2.lag) - 1 | 0 | 0 | mkt + eventTime + time, data = meanStatsTime);
regr_0.1     = felm(scale(dPI)     ~ scale(dPI.lag) + scale(dED_0.1.lag) - 1 | 0 | 0 | mkt + eventTime + time, data = meanStatsTime);
regr_0.1.rev = felm(scale(dED_0.1) ~ scale(dPI.lag) + scale(dED_0.1.lag) - 1 | 0 | 0 | mkt + eventTime + time, data = meanStatsTime);
regr_all     = felm(scale(dPI)     ~ scale(dPI.lag) + scale(dED_all.lag) - 1 | 0 | 0 | mkt + eventTime + time, data = meanStatsTime);
regr_all.rev = felm(scale(dED_all) ~ scale(dPI.lag) + scale(dED_all.lag) - 1 | 0 | 0 | mkt + eventTime + time, data = meanStatsTime);
# With one lag, OLS estimation and t-tests of the VAR model is fine: https://stats.stackexchange.com/questions/286656/granger-causality-with-or-without-var 
capture.output(stargazer(regr_0.2, regr_0.2.rev, regr_0.1, regr_0.1.rev, regr_all, regr_all.rev, title="Time series regressions", align=TRUE, 
                         dep.var.labels=c("dPI","dED_0.2","dPI","dED_0.1","dPI","dED_all"),
                         covariate.labels=c("$\\Delta$ \\textit{PriceInefficiency}",
                                            "$\\Delta$ Nr. of edges ($\\rho>0.2$)","$\\Delta$ Nr. of edges ($\\rho>0.1$)",
                                            "$\\Delta$ Sum of edge weights ($\\Sigma \\rho$)"),
                         omit.stat=c("ser","adj.rsq"), no.space=TRUE, star.cutoffs = c(1,0.10,0.05), star.char = c(" ","*","**"), omit.table.layout = "n", header = F, digits = 2, type = "text"), 
               file = "output/Table4.txt");

# Table 4X: Same as above but including EBS
meanStatsTime = subsampleStats[eventTime %in% 1:5,list(dPI     = mean(dPI,na.rm=T),     dPI.lag     = mean(dPI.lag,na.rm=T),
                                                       dED_0.1 = mean(dED_0.1,na.rm=T), dED_0.1.lag = mean(dED_0.1.lag,na.rm=T),
                                                       dED_0.2 = mean(dED_0.2,na.rm=T), dED_0.2.lag = mean(dED_0.2.lag,na.rm=T),
                                                       dED_all = mean(dED_all,na.rm=T), dED_all.lag = mean(dED_all.lag,na.rm=T)),by=list(mkt,eventTime,time)]
regr_0.2     = felm(scale(dPI)     ~ scale(dPI.lag) + scale(dED_0.2.lag) - 1 | 0 | 0 | mkt + eventTime + time, data = meanStatsTime);
regr_0.2.rev = felm(scale(dED_0.2) ~ scale(dPI.lag) + scale(dED_0.2.lag) - 1 | 0 | 0 | mkt + eventTime + time, data = meanStatsTime);
regr_0.1     = felm(scale(dPI)     ~ scale(dPI.lag) + scale(dED_0.1.lag) - 1 | 0 | 0 | mkt + eventTime + time, data = meanStatsTime);
regr_0.1.rev = felm(scale(dED_0.1) ~ scale(dPI.lag) + scale(dED_0.1.lag) - 1 | 0 | 0 | mkt + eventTime + time, data = meanStatsTime);
regr_all     = felm(scale(dPI)     ~ scale(dPI.lag) + scale(dED_all.lag) - 1 | 0 | 0 | mkt + eventTime + time, data = meanStatsTime);
regr_all.rev = felm(scale(dED_all) ~ scale(dPI.lag) + scale(dED_all.lag) - 1 | 0 | 0 | mkt + eventTime + time, data = meanStatsTime);
# With one lag, OLS estimation and t-tests of the VAR model is fine: https://stats.stackexchange.com/questions/286656/granger-causality-with-or-without-var 
capture.output(stargazer(regr_0.2, regr_0.2.rev, regr_0.1, regr_0.1.rev, regr_all, regr_all.rev, title="Time series regressions", align=TRUE, 
                               dep.var.labels=c("dPI","dED_0.2","dPI","dED_0.1","dPI","dED_all"),
                               covariate.labels=c("$\\Delta$ \\textit{PriceInefficiency}",
                                                  "$\\Delta$ Nr. of edges ($\\rho>0.2$)","$\\Delta$ Nr. of edges ($\\rho>0.1$)",
                                                  "$\\Delta$ Sum of edge weights ($\\Sigma \\rho$)"),
                               omit.stat=c("ser","adj.rsq"), no.space=TRUE, star.cutoffs = c(1,0.10,0.05), star.char = c(" ","*","**"), omit.table.layout = "n", header = F, digits = 2, type = "text"), 
               file = "output/Table4_EBS.txt");

# Table 5: News vs NoNews (matched subsamples conditional on top-quartile realized volatility)

# News analysis with double clustered standard errors
subsampleStats$News = subsampleStats$newsMatched == "News";
subsampleStats$sqrtRV = sqrt(subsampleStats$RV);
variables = c("PI","speed","ED_0.2","ED_0.1","ED_all","sqrtRV");
Table5 = matrix(ncol=12,nrow=length(variables)+1,dimnames=list(c("PriceInefficiency","InfoSpeed","# edges > 0.2","# edges > 0.1","Sum of edge weights","realized volatility","N"),rep(c("News","NoNews","Diff","p"),3)));

subset = !is.na(subsampleStats$News) & subsampleStats$eventTime==5;
Table5[length(variables)+1,1:2] = table(subsampleStats$News[subset]);
regr = felm(PI + speed + ED_0.1 + ED_0.2 + ED_all + sqrtRV ~ News | 0 | 0 | mkt + subsample, data = subsampleStats, subset = subset);
subset = !is.na(subsampleStats$News) & subsampleStats$eventTime==5 & subsampleStats$mkt=="EBS";
Table5[length(variables)+1,5:6] = table(subsampleStats$News[subset]);
regrEBS = felm(PI + speed + ED_0.1 + ED_0.2 + ED_all + sqrtRV ~ News, data = subsampleStats, subset = subset);
subset = !is.na(subsampleStats$News) & subsampleStats$eventTime==5 & subsampleStats$mkt!="EBS";
Table5[length(variables)+1,9:10] = table(subsampleStats$News[subset]);
regrOTC = felm(PI + speed + ED_0.1 + ED_0.2 + ED_all + sqrtRV ~ News | 0 | 0 | mkt + subsample, data = subsampleStats, subset = subset);

for(v in 1:length(variables)){
  Table5[v,1:4] =   c(sum(summary(regr,lhs=variables[v])$coeff[1:2,1]),
                      summary(regr,lhs=variables[v])$coeff[1,1],
                      summary(regr,lhs=variables[v])$coeff[2,c(1,4)])
  Table5[v,4+1:4] = c(sum(summary(regrEBS,lhs=variables[v])$coeff[1:2,1]),
                      summary(regrEBS,lhs=variables[v])$coeff[1,1],
                      summary(regrEBS,lhs=variables[v])$coeff[2,c(1,4)])
  Table5[v,8+1:4] = c(sum(summary(regrOTC,lhs=variables[v])$coeff[1:2,1]),
                      summary(regrOTC,lhs=variables[v])$coeff[1,1],
                      summary(regrOTC,lhs=variables[v])$coeff[2,c(1,4)])
}
capture.output(round(Table5,2), file = "output/Table5.txt")

# Figures 4 & 5: Scatter plots of market means
subset = subsampleStats$eventTime==5 & subsampleStats$mkt != "EBS";
mktMeans = subsampleStats[subset,list(PI=mean(PI),ED_all=mean(ED_all),ED_0.1=mean(ED_0.1),ED_0.2=mean(ED_0.2),relSpr=mean(relSpr),ISl=mean(ISl)*100),by=list(mkt)];
mktMeans$DLR = paste0("DLR",match(mktMeans$mkt,nlab))

Figure4A = ggplot(data = mktMeans, aes(x=ED_0.2,y=PI)) + 
  theme_classic(base_size = 16) +
  ylab("Dealer price inefficiency") + xlab("Dealer centrality") +
  geom_point(size=5) +    # Use hollow circles
  geom_smooth(method=lm,se =T) +   # Add linear regression line 
  geom_label_repel(aes(label = DLR), label.size = NA, fill = NA)
ggsave("output/Fig4A.pdf", h=4,w=4, Figure4A)

Figure4B = ggplot(data = mktMeans, aes(x=ED_0.2,y=ISl)) + 
  theme_classic(base_size = 16) +
  ylab("Dealer information share (%)") + xlab("Dealer centrality") +
  geom_point(size=5) +    # Use hollow circles
  geom_smooth(method=lm,se =T) +   # Add linear regression line 
  geom_label_repel(aes(label = DLR), label.size = NA, fill = NA)
ggsave("output/Fig4B.pdf", h=4,w=4, Figure4B)

Figure5 = ggplot(data = mktMeans, aes(x=ED_0.2,y=relSpr)) + 
  theme_classic(base_size = 16) +
  ylab("Relative bid-ask spread (bps)") + xlab("Dealer centrality") +
  geom_point(size=5) +    # Use hollow circles
  geom_smooth(method=lm,se =T) +   # Add linear regression line 
  geom_label_repel(aes(label = DLR), label.size = NA, fill = NA)
ggsave("output/Fig5.pdf", h=4,w=4, Figure5)

# Robustness wrt centrality measure used
Figure4A_ED0.1 = ggplot(data = mktMeans, aes(x=ED_0.1,y=PI)) + 
  theme_classic(base_size = 16) +
  ylab("Dealer price inefficiency") + xlab("Dealer centrality") +
  geom_point(size=5) +    # Use hollow circles
  geom_smooth(method=lm,se =T) +   # Add linear regression line 
  geom_label_repel(aes(label = DLR), label.size = NA, fill = NA)
ggsave("output/Fig4A_ED0.1.pdf", h=4,w=4, Figure4A_ED0.1)

Figure4B_ED0.1 = ggplot(data = mktMeans, aes(x=ED_0.1,y=ISl)) + 
  theme_classic(base_size = 16) +
  ylab("Dealer information share (%)") + xlab("Dealer centrality") +
  geom_point(size=5) +    # Use hollow circles
  geom_smooth(method=lm,se =T) +   # Add linear regression line 
  geom_label_repel(aes(label = DLR), label.size = NA, fill = NA)
ggsave("output/Fig4B_ED0.1.pdf", h=4,w=4, Figure4B_ED0.1)

Figure5_ED0.1 = ggplot(data = mktMeans, aes(x=ED_0.1,y=relSpr)) + 
  theme_classic(base_size = 16) +
  ylab("Relative bid-ask spread (bps)") + xlab("Dealer centrality") +
  geom_point(size=5) +    # Use hollow circles
  geom_smooth(method=lm,se =T) +   # Add linear regression line 
  geom_label_repel(aes(label = DLR), label.size = NA, fill = NA)
ggsave("output/Fig5_ED0.1.pdf", h=4,w=4, Figure5_ED0.1)

Figure4A_EDall = ggplot(data = mktMeans, aes(x=ED_all,y=PI)) + 
  theme_classic(base_size = 16) +
  ylab("Dealer price inefficiency") + xlab("Dealer centrality") +
  geom_point(size=5) +    # Use hollow circles
  geom_smooth(method=lm,se =T) +   # Add linear regression line 
  geom_label_repel(aes(label = DLR), label.size = NA, fill = NA)
ggsave("output/Fig4A_EDall.pdf", h=4,w=4, Figure4A_EDall)

Figure4B_EDall = ggplot(data = mktMeans, aes(x=ED_all,y=ISl)) + 
  theme_classic(base_size = 16) +
  ylab("Dealer information share (%)") + xlab("Dealer centrality") +
  geom_point(size=5) +    # Use hollow circles
  geom_smooth(method=lm,se =T) +   # Add linear regression line 
  geom_label_repel(aes(label = DLR), label.size = NA, fill = NA)
ggsave("output/Fig4B_EDall.pdf", h=4,w=4, Figure4B_EDall)

Figure5_EDall = ggplot(data = mktMeans, aes(x=ED_all,y=relSpr)) + 
  theme_classic(base_size = 16) +
  ylab("Relative bid-ask spread (bps)") + xlab("Dealer centrality") +
  geom_point(size=5) +    # Use hollow circles
  geom_smooth(method=lm,se =T) +   # Add linear regression line 
  geom_label_repel(aes(label = DLR), label.size = NA, fill = NA)
ggsave("output/Fig5_EDall.pdf", h=4,w=4, Figure5_EDall)
