# This script draws the network plots of Fig.3 and creates Table 1 and Table 3B

# Change the following to the appropriate working directory
setwd("D:/informationrevelation/code/analysis/");

# Loading required R packages
require(data.table);
require(dplyr);
require(igraph);
require(ggraph);
require(ggrepel);
require(ggforce);
require(extrafont);
font_import(paths = NULL, recursive = TRUE, prompt = TRUE,pattern = NULL)
extrafont::loadfonts(device="win")

# Loading subsample network stats (from script 1g)
load(file="input/networks_data_J10000_K100");

N = length(nlab);
significanceLevel = 0.05;
thresholds = c(1 - (1-0.05)^(1/(N*(N-1)/2)),1 - (1-0.10)^(1/(N*(N-1)/2)));
names(thresholds) = c("5%","10%");
mktOrdering = match(c("EBS","BBBB","DCFX","PEP0","FXN0","PFD0","DUBA","TFS0","WBRS"),nlab);
nlab2 = c("EBS",paste0("DLR",1:8));
nlab3 = c("EBS",paste0("DLR",match(nlab[mktOrdering][2:9],nlab)));

# Function for reporting mean network properties across subsamples
meanNet = function(rows,cols){
  means = rep(NA,length(cols));
  pvals = rep(NA,length(cols));
  for(i in cols){
    if(var(networks[rows,i,with=F],na.rm=T)>0){
      tmp = t.test(networks[rows,i,with=F]);
      means[i-min(cols)+1] = tmp$estimate;
      pvals[i-min(cols)+1] = tmp$p.value;
    } else {
      means[i-min(cols)+1] = mean(unlist(networks[rows,i,with=F]),na.rm=T);
    }
  }
  out = c(means,pvals)
  return(out);
}

# Time limits
tauPeriods = 0:50; # Network snapshots

# Subsample sets
subsets = list(rep(T,nrow(networks)),
               networks$subsample <= 480,
               networks$subsample >  480)
subsetLabels = c("All","Week1","Week2");

AdjMatrixTable = vector("list",length=length(subsets))

for(s in 1:length(subsets)){
  subset = which(subsets[[s]] & networks$eventTime %in% tauPeriods);
  
  # Adjacency matrix averaged across subsamples
  edges = aggregate(subset, by = list(networks$eventTime[subset]), meanNet, cols=2+1:(2*(N^2)));
  dimnames(edges$x)[[2]] = paste0(dimnames(networks)[[2]][2+1:(2*(N^2))],rep(c(".m",".p"),each = 2*(N^2)))

  # Price inefficiency averaged across subsamples
  distance = aggregate(subset, by = list(networks$eventTime[subset]), meanNet, cols=(2+2*(N^2))+1:N);
  dimnames(distance$x)[[2]] = paste0(dimnames(networks)[[2]][(2+2*(N^2))+1:N],rep(c(".m",".p"),each = N))
  
  PI = vector("list",length=N);
  for(i in 1:N){
    PI[[i]] = data.table(eventTime = distance$Group.1, D = distance$x[,paste0("PI.",nlab[i],".m")],mkt=nlab[i])
  }
  PI = do.call(rbind,PI);

  # Network plots
  for(tau in tauPeriods){
    if(s == 1 | tau == 5){
      # Price inefficiency --> vertex coordinates
      PI.tau       = PI$D[PI$eventTime==tau][mktOrdering];
      tmp = max(c(PI.tau,1))+0.1;
      coordinates     = data.frame(x = c(tmp,-tmp,0,PI.tau * sin(2 * pi * ((0:(N - 1))/N))), y = c(tmp,-tmp,0,PI.tau * cos(2 * pi * ((0:(N - 1))/N))));
      
      # Adjacency matrix
      adjMatrix      = matrix(edges$x[edges$Group.1 == as.character(tau),1:(N^2)],        N,N, dimnames= list(nlab,nlab));
      adjMatrix.p    = matrix(edges$x[edges$Group.1 == as.character(tau),2*(N^2)+1:(N^2)],N,N);
      adjMatrix      = rbind(matrix(0,3,12),cbind(matrix(0,9,3),adjMatrix[mktOrdering,mktOrdering]))
      adjMatrix.p    = rbind(matrix(0,3,12),cbind(matrix(0,9,3),adjMatrix.p[mktOrdering,mktOrdering]))
      
      # Clockwise numbering of dealers
      dimnames(adjMatrix) = list(c("",""," ",nlab3),c("A","B"," ",nlab3))
      dimnames(adjMatrix.p) = list(c("",""," ",nlab3),c("A","B"," ",nlab3))
    }
      
    # Exporting adjacency matrix for tau = 5
    if(tau==5){
      AdjMatrixTable[[s]] = list(adjMatrix = adjMatrix[nlab2,nlab2], p.values = round(adjMatrix.p[nlab2,nlab2],5),significance.thresholds = thresholds);
    }
      
    if(s==1){
      # Structuring data for plotting and 
      # determining what edges should be plotted (those that are statistically significant)
      cors = data.frame(x = dimnames(adjMatrix)[[2]][combn(1:(N+3),2)[1,]], y = dimnames(adjMatrix)[[2]][combn(1:(N+3),2)[2,]], r = as.vector(adjMatrix[lower.tri(adjMatrix)]), p = as.vector(adjMatrix.p[lower.tri(adjMatrix.p)]));
      cors$r[cors$p > thresholds[1]] = 0;
      graph_cors = graph_from_data_frame(as.tbl(cors), directed = FALSE);

      # Vertex label coordinates
      label.coord = data.frame(x = c(0,0,0,0.165 * sin(2 * pi * ((0:(N - 1))/N))), y = c(0,0,0,0.165 * cos(2 * pi * ((0:(N - 1))/N))));
      label.coord[3+1:N,][PI.tau>0.8,] = data.frame(x = c(0,0,0,-0.165 * sin(2 * pi * ((0:(N - 1))/N))), y = c(0,0,0,-0.165 * cos(2 * pi * ((0:(N - 1))/N))))[3+1:N,][PI.tau>0.8,];
      
      if(tau %in% c(0:5,10,20,50)){
        # Plotting
        netPlot = ggraph(graph_cors, layout = "manual", node.positions = coordinates) +
          geom_edge_link(aes(edge_alpha = abs(r), edge_width = abs(r), color = r)) +
          guides(edge_alpha = "none", edge_width = "none", color = "none") +
          scale_edge_colour_gradient2(limits = c(-1, 1), low = "firebrick2", mid = "white", high = "dodgerblue2", guide="none") +
          geom_node_point(color = c("white","white","red",rep("gray",N)), size = c(rep(0,2),3,rep(5,N)), shape = c(0,0,18,rep(19,N))) +
          geom_node_text(aes(label = name), repel = F, color = c(rep("white",3),rep("black",N)), nudge_x = label.coord$x, nudge_y = label.coord$y) +
          theme_graph() + 
          geom_circle(aes(r=1,y0=0,x0=0), col = "red", linetype = "dashed") +
          theme(plot.margin=grid::unit(c(0.1,0.1,0.1,0.1), "mm"))
        
        ggsave(paste0("output/Fig3_tau",tau,".pdf"), h=4,w=4, netPlot)
      }

      # Network plots for movies (not in paper, but needed for the slides)
      label.coord = data.frame(x = c(0,0,0,0.1 * sin(2 * pi * ((0:(N - 1))/N))), y = c(0,0,0,0.1 * cos(2 * pi * ((0:(N - 1))/N))));
      netPlot = ggraph(graph_cors, layout = "manual", node.positions = coordinates) +
        geom_edge_link(aes(edge_alpha = abs(r), edge_width = abs(r), color = r)) +
        guides(edge_alpha = "none", edge_width = "none") +
        scale_edge_colour_gradient2(limits = c(-1, 1), low = "firebrick2", mid = "white", high = "dodgerblue2") +
        geom_node_point(color = c("white","white","red",rep("black",N)), size = c(rep(0,2),3,rep(5,N)), shape = c(0,0,18,rep(19,N))) +
        geom_node_text(aes(label = name), repel = F, color = c(rep("white",3),rep("black",N)), nudge_x = label.coord$x, nudge_y = label.coord$y) +
        theme_graph() +
        theme(plot.title = element_text(hjust = 0.5),plot.subtitle = element_text(hjust = 0.5)) +
        labs(title = paste0(if(s>1) subsetLabels[s] else "Baseline sample",": Event time ",tau)) +
        geom_circle(aes(r=1,y0=0,x0=0), col = "red", linetype = "dashed")
      
      ggsave(paste0("slides output/movie/CIRnetwork_",subsetLabels[s],"_pcor_",if(tau<1000) "0",if(tau<100) "0",if(tau<10) "0",tau,".pdf"), h=9,w=10, netPlot)
    }
  }
  print(subsetLabels[s]);
}

# Plot of network just before the shock (for the slides)
coordinates     = data.frame(x = c(1,-1,0,rep(1,N) * sin(2 * pi * ((0:(N - 1))/N))), y = c(tmp,-tmp,0,rep(1,N) * cos(2 * pi * ((0:(N - 1))/N))));
label.coord = data.frame(x = c(0,0,0,0.1 * sin(2 * pi * ((0:(N - 1))/N))), y = c(0,0,0,0.1 * cos(2 * pi * ((0:(N - 1))/N))));
netPlot = ggraph(graph_cors, layout = "manual", node.positions = coordinates) +
  geom_node_point(color = c("white","white","red",rep("black",N)), size = c(rep(0,2),3,rep(5,N)), shape = c(0,0,18,rep(19,N))) +
  geom_node_text(aes(label = name), repel = F, color = c(rep("white",3),rep("black",N)), nudge_x = label.coord$x, nudge_y = label.coord$y) +
  theme_graph() +
  theme(plot.title = element_text(hjust = 0.5),plot.subtitle = element_text(hjust = 0.5)) +
  labs(title = "Baseline sample: Event time 0-") +
  geom_circle(aes(r=1,y0=0,x0=0), col = "red", linetype = "dashed")
ggsave(paste0("slides output/movie/just_before.pdf"), h=9,w=10, netPlot)


# Table 1: Adjacency matrix for both weeks
Table1 = round(AdjMatrixTable[[1]]$adjMatrix,2)
Table1_stars = matrix("",9,9,dimnames=dimnames(Table1))
Table1_stars[AdjMatrixTable[[1]]$p.values < AdjMatrixTable[[1]]$significance.thresholds[2]] = "*";
Table1_stars[AdjMatrixTable[[1]]$p.values < AdjMatrixTable[[1]]$significance.thresholds[1]] = "**";
capture.output(list(edges = Table1, significance = Table1_stars), file = "output/Table1.txt")

# Correlations of adjacency matrices across week1, week2, and conditional on high IS
adjMatrix_0.1 = matrix(nrow=N*(N-1)/2,ncol=2,dimnames=list(NULL,c("Week1","Week2")));
adjMatrix_0.2 = matrix(nrow=N*(N-1)/2,ncol=2,dimnames=list(NULL,c("Week1","Week2")));
adjMatrix_all = matrix(nrow=N*(N-1)/2,ncol=2,dimnames=list(NULL,c("Week1","Week2")));
for(s in 1:2){
  adjMatrix_all[,s] = as.vector(AdjMatrixTable[[s+1]]$adjMatrix[lower.tri(AdjMatrixTable[[s+1]]$adjMatrix)]);
  adjMatrix.p       = as.vector(AdjMatrixTable[[s+1]]$p.values[lower.tri(AdjMatrixTable[[s+1]]$adjMatrix)]);

  adjMatrix_all[adjMatrix_all[,s] <  0, s]   = 0;
  adjMatrix_0.1[adjMatrix_all[,s] <  0.1, s] = 0;
  adjMatrix_0.1[adjMatrix_all[,s] >= 0.1, s] = 1;
  adjMatrix_0.2[adjMatrix_all[,s] <  0.2, s] = 0;
  adjMatrix_0.2[adjMatrix_all[,s] >= 0.2, s] = 1;
}
Table3B = matrix(c(cor(adjMatrix_0.2)[2],
                   cor(adjMatrix_0.1)[2],
                   cor(adjMatrix_all)[2]), 3, 1);
dimnames(Table3B) = list(c("All edge weights","Centraltity0.2","Centraltity0.1"), "Corr Week1--Week2")
cor.test(adjMatrix_0.1[,1],adjMatrix_0.1[,2])
capture.output(round(Table3B,2), file = "output/Table3B.txt")

# Exporting adjacency matrix for comparison to CHF crash
save(list=c("AdjMatrixTable"),file="temp/adjMatrix");

