# This script generates Figure 7

# Change the following to the appropriate working directory
setwd("D:/informationrevelation/code/analysis/");

# Loading required R packages
require(data.table);
require(zoo);

# Loading PIPE functions
startTime = 8*3600;  # 9 AM Swiss time
endTime   = 16*3600; # 5 PM Swiss time

# Reading OTC data
load(file = "input/dealer_pseudodata");
setnames(raw,c("dateRaw","timeRaw","bid","ask","mkt","score"));
otc      = raw[raw$dateRaw=="15-01-2015"];
otc      = otc[otc$score>0.9];
otc$mid  = (otc$bid + otc$ask) / 2;
time     = strptime(otc$timeRaw,format="%H:%M:%OS")
otc$time = time$hour*3600 + time$min*60 + time$sec
otc      = otc[otc$time > startTime & otc$time <= (endTime + 60),]

# OTC market quotes minute-by-minute
# The reported numbers are medians across all quotes posted in that particular minute
# The motivation is that "no quote update" does not mean that the previous quote is still valid. 
# Carrying quotes forward would make it look like all markets are open throughout the crash, with the current prices being their last posted prices
# Note that these stats are across all OTC dealers, not only our subsample of eight dealers
otcQuoteStats = function(rows){
  return(c(median(otc$ask[rows]/2+otc$bid[rows]/2),
           median(10000*(log(otc$ask[rows])-log(otc$bid[rows]))/2),
           length(rows)))
}
OTC.quotes = aggregate(1:nrow(otc),by=list(findInterval(otc$time,seq(startTime,endTime,60))),otcQuoteStats)

# Reading EBS data
#raw = read.csv(gzfile("input/ICAP data/ehd_5p0-20150115.ccy.out.gz"), as.is=TRUE, header=FALSE);
load(file = "input/interdealer_pseudodata");

# EBS quote analysis
# To be consistent with the OTC analysis we use the median of all quotes in each minute
ebs      = raw[raw[,4]=="Q" & raw[,3]=="EUR/CHF" & raw[,6]==1,-c(3,6)];
dimnames(ebs)[[2]] = c("rawdate","rawtime","type","side","price","vol","nq")

# Because EBS is a LOB market, posted quotes are valid until executed or cancelled
# Carrying quotes forward is thus valid for EBS
ebs$bid  = NA
ebs$bid[ebs$side==0] = ebs$price[ebs$side==0]
ebs$bid  = na.locf(ebs$bid,na.rm=F)
ebs$ask  = NA
ebs$ask[ebs$side==1] = ebs$price[ebs$side==1]
ebs$ask  = na.locf(ebs$ask,na.rm=F)
ebs$mid  = (ebs$bid + ebs$ask)/2;
ebs$spr  = 10000*(log(ebs$ask) - log(ebs$bid))/2;
ebs$sprCap = ebs$spr; 
ebs$sprCap[ebs$spr > 50] = NA; 
ebs      = ebs[ebs$spr >= 0 & !is.na(ebs$spr) & c(1,diff(ebs$time)>0),]

# Relevant time frame
time     = strptime(ebs$rawtime,format="%H:%M:%OS")
ebs$time = time$hour*3600 + time$min*60 + time$sec
ebs      = ebs[ebs$time>startTime & ebs$time <= (endTime+60),]

# Median midpoint and spread (for consistency with OTC reports)
ebsQuoteStats = function(rows){
  return(c(median(ebs$mid[rows],na.rm=T),
           median(ebs$spr[rows],na.rm=T),
           median(ebs$sprCap[rows],na.rm=T),
           length(rows)))
}
EBS.quotes = aggregate(1:nrow(ebs),by=list(findInterval(ebs$time,seq(startTime,endTime,60))),ebsQuoteStats)

# EBS trade analysis
ebs      = raw[raw$V4=="D" & raw$V3=="EUR/CHF",-c(3,4,6)];
dimnames(ebs)[[2]] = c("rawdate","rawtime","side","price","vol","nfirm")
time     = strptime(ebs$rawtime,format="%H:%M:%OS")
ebs$time = time$hour*3600 + time$min*60 + time$sec
ebs      = ebs[ebs$time> startTime & ebs$time <= (endTime+60),]

ebsTradeStats = function(rows){
  return(c(-sum(as.numeric(as.character(ebs$vol[rows][ebs$side[rows]==0])) * as.numeric(as.character(ebs$price[rows][ebs$side[rows]==0]))),
            sum(as.numeric(as.character(ebs$vol[rows][ebs$side[rows]==1])) * as.numeric(as.character(ebs$price[rows][ebs$side[rows]==1]))))/1e6)  
}
EBS.trades = aggregate(1:nrow(ebs),by=list(findInterval(ebs$time,seq(startTime,endTime,60))),ebsTradeStats)

# Merging the minute-by-minute statistics for output
EBS = merge(EBS.quotes,EBS.trades,by=1,all=TRUE)
EBS = cbind(EBS[,1],EBS[,2],EBS[,3])
out = merge(EBS,OTC.quotes,by=1,all=TRUE)
out = cbind(seq(startTime,endTime,60),out[,2:7],out[,8])
dimnames(out)[[2]] = c("time","EBSmid","EBSspr","EBSsprCap","EBSnq","EBSsell","EBSbuy","OTCmid","OTCspr","OTCnq")
out[is.na(out[,"EBSbuy"]),c("EBSbuy","EBSsell")] = 0;
write.csv(out,file="output/CHFcrashData.csv")

# Plots for Figure 7
pdf('output/Fig7a.pdf')
plot(out$OTCnq,type="l", col="blue")
points(out$EBSnq,type="l")
dev.off()

pdf('output/Fig7b_price.pdf')
plot(out$EBSmid,type="l")
dev.off()

pdf('output/Fig7b_tradingvol.pdf')
plot(out$EBSbuy*out$EBSmid,type="h", ylim = c(-1000,1000),lwd=1)
points(out$EBSsell*out$EBSmid,type="h", col = "blue",lwd=1)
dev.off()

pdf('output/Fig7c.pdf')
plot(out$EBSspr,type="l")
points(out$OTCspr,type="l", col = "blue",lwd=1)
dev.off()

pdf('output/Fig7c_cap.pdf')
plot(out$EBSsprCap,type="l")
points(out$OTCspr,type="l", col = "blue",lwd=1)

pdf('output/Fig7c_log.pdf')
plot(out$EBSspr,type="l", log = "y")
points(out$OTCspr,type="l", col = "blue",lwd=1)
dev.off()
