# This script estimates the VECM and reports corresponding network characteristics

# Change the following to the appropriate working directory
setwd("D:/informationrevelation/code/build/");

# Loading functions required for the VECM estimation
source("code/VECM functions.R");

# Preamble
freq = c(100,5*60); 
# freq[1] is the sampling frequency, expressed in milliseconds
# freq[2] is the subsample length, expressed in seconds
startTime = 8*3600;
endTime = 16*3600;
dates     = c(20150119:20150123,20150126:20150130);

# Loading and merging Olsen and ICAP data
load(file="temp/dealer_quotes");
load(file="temp/EBS_quotes");
nlab = c(brokers,"EBS");
quotes[[9]] = as.matrix(qq[,1:3])
N = length(nlab);
rm(qq);

# VECM estimation inputs
J         = 10000; # Number of iterations to determine convergence of IRF and to obtain information shares
K         = 100;   # Maximum horizon for obtaining partial correlation network and distance
L         = NA;    # VECM lag length; when NA a lag selection criterion is used

# Number of subsamples per day
rpd       = (endTime-startTime)/freq[2]; 

# Output objects:
# vecm is a list holding the VECM estimates and network objects for each subsample
# nq1 reports the number of quotes available in the data for each subsample and each market
# nq2 reports the number of quote price changes at the chosen sampling frequency, for each subsample and each market
# only markets with nq2 > 10 are included in the estimation
vecm      = vector("list",length=length(dates)*rpd);
nq1       = matrix(nrow=rpd*length(dates),ncol=N,dimnames=list(NULL,nlab)); 
nq2       = matrix(nrow=rpd*length(dates),ncol=N,dimnames=list(NULL,nlab));

for(day in 1:length(dates)){
  tmp = Sys.time()
  for(ff in 1:rpd){
    for(i in 1:N){
      nq1[(day-1)*rpd+ff,i] = length(quotes[[i]][quotes[[i]][,"time"]>(startTime+(ff-1)*freq[2]) & quotes[[i]][,"time"]<(startTime+ff*freq[2]) & quotes[[i]][,"date"]==dates[day],])/3
    }
    tmp2 = Sys.time()

    # Retrieving price series on the chosen sampling frequency
    p = VECM.freq(firstSec=startTime+(ff-1)*freq[2],lastSec=startTime+ff*freq[2],dataFreqMs=freq[1],dateCurrent=dates[day]);
    if(ncol(p)>1){
      nq2[(day-1)*rpd+ff,nlab %in% dimnames(p)[[2]]] = colSums(p[2:nrow(p),]!=p[1:(nrow(p)-1),],na.rm=T)
      p = p[,colSums(p[2:nrow(p),]!=p[1:(nrow(p)-1),],na.rm=T) > 10]; # Excludes markets with low nq2
    } 
    # Estimating the VECM in case there is data for more than one market
    if(is.data.frame(p)){
      vecm[[(day-1)*rpd+ff]] = estimateVECM(fileName=paste0("temp/VECM_",dates[day],"_",freq[1],"ms_",ff,"_",rpd,if(is.na(L)) "" else paste0("_",L,"lags")),p=p, L=L, J=J, K=K, nlab = nlab);
    } 
    if(day==1){
      print(Sys.time()-tmp2)
      print(ff);
    } 
  }
  print(Sys.time()-tmp)
  print(dates[day])
  save(list=c("vecm","dates","N","freq","startTime","endTime","rpd","nlab","nq1","nq2","J","K"),file=paste0("temp/VECM_",freq[1],"ms_",freq[2]/60,"min_",N,"markets_J",J,"K",K,if(is.na(L)) "" else paste0("_",L,"lags"),"_2"));
}
  
# Estimation statistics reported in Footnote 15
lagSelect = matrix(nrow=length(dates)*rpd, ncol=6, dimnames=list(NULL,c("AIC(n)","HQ(n)","SC(n)","FPE(n)","L","N")));
convergence = vector(length=length(dates)*rpd);
for(day in 1:length(dates)){
  for(ff in 1:rpd){
    if(length(vecm[[(day-1)*rpd+ff]])>1){
      lagSelect[(day-1)*rpd+ff,] = c(vecm[[(day-1)*rpd+ff]]$lagIC,vecm[[(day-1)*rpd+ff]]$L,vecm[[(day-1)*rpd+ff]]$N);
      convergence[(day-1)*rpd+ff] = sum(vecm[[(day-1)*rpd+ff]]$conv>0.10) == 0;
    }
  }
}

mean(rowSums(mktSelect)==9); # 91%
mean(rowSums(mktSelect)<8);  # 2%
sum(!convergence);           # 11 out of 960
