# This script estimates the VECM for the CHF crash day, and runs a bootstrap procedure to obtain the information network characteristics

# Change the following to the appropriate working directory
setwd("D:/informationrevelation/code/build/");

# Loading functions and R packages required for the VECM estimation
source("code/VECM functions.R");
require(data.table);

# Preamble
freq = c(100,5*60); 
# freq[1] is the sampling frequency, expressed in milliseconds
# freq[2] is the subsample length, expressed in seconds
startTime = 10.5*3600;
endTime = 16*3600;

# Loading and merging Olsen and ICAP data
load(file="temp/dealer_quotes");
load(file="temp/EBS_quotes");
nlab = c(brokers,"EBS");
quotes[[9]] = as.matrix(qq[,1:3])
N = length(nlab);
rm(qq);

# VECM estimation inputs
J         = 1000; # Number of iterations to determine convergence of IRF and to obtain information shares
K         = 100;   # Maximum horizon for obtaining partial correlation network and distance
L         = NA; # Lag length; when NA a lag selection criterion is used

# Number of subsamples per day
rpd       = (endTime-startTime)/freq[2];

# VECM output objects:
vecm      = vector("list",length=rpd);

# CHF crash: estimation with set number of markets, as follows:
# 11:30 - 11:40 (1-2),   3 markets: "DUBA","WBRS","EBS"
# 11:45 - 12:10 (4-8),   4 markets: "DUBA","PFD0","WBRS","EBS"
# 12:10 - 12:30 (9-12),  6 markets: "FXN0","DUBA","PFD0","WBRS","BBBB","EBS"
# 12:30 - 13:15 (13-21), 8 markets: "FXN0","DCFX","TFS0","DUBA","PFD0","WBRS","PEP0","EBS"
# 13:15 - 14:15 (22-33), 8 markets: "FXN0","DCFX","TFS0","DUBA","PFD0","WBRS","BBBB","EBS"
# 14:15 - 15:15 (34-45), 9 markets
# 15:15 - 16:15 (46-57), 9 markets
# 16:15 - 17:00 (58-66), 9 markets

subsetTimes = cbind(startTime+(c(1,4,9,13,22,34,46,58)-1)*freq[2],startTime+c(2,8,12,21,33,45,57,66)*freq[2]);
dimnames(subsetTimes) = list(c("1130_1140","1145_1210","1210_1230","1230_1315","1315_1415","1415_1515","1515_1615","1615_1700"),c("start","end"));
subsetMkts = list(c("DUBA","WBRS","EBS"),
                  c("DUBA","PFD0","WBRS","EBS"),
                  c("FXN0","DUBA","PFD0","WBRS","BBBB","EBS"),
                  c("FXN0","DCFX","TFS0","DUBA","PFD0","WBRS","PEP0","EBS"),
                  c("FXN0","DCFX","TFS0","DUBA","PFD0","WBRS","BBBB","EBS"),
                  nlab,
                  nlab,
                  nlab);

for(s in 1:nrow(subsetTimes)){
  # Obtaining equi-spaced price observations
  p = VECM.freq(firstSec=subsetTimes[s,1],lastSec=subsetTimes[s,2],dataFreqMs=freq[1],dateCurrent = 20150115, markets = which(nlab %in% subsetMkts[[s]]));

  # Estimating the VECM and obtaining network metrics
  vecm[[s]] = estimateVECM(fileName=paste0("temp/VECM_Jan15_",dimnames(subsetTimes)[[1]][s],if(is.na(L)) "" else paste0("_",L,"lags")), p=p, L=L, J=J, K=K, nlab = nlab);
  print(dimnames(subsetTimes)[[1]][s]);
}
save(list=c("vecm","N","freq","subsetMkts","subsetTimes","nlab","J","K"),file=paste0("temp/VECM_Jan15",freq[1],"ms_",freq[2]/60,"min_",N,"markets_J",J,"K",K,if(is.na(L)) "" else paste0("_",L,"lags")));
#load(file=paste0("temp/VECM_Jan15",freq[1],"ms_",freq[2]/60,"min_",N,"markets_J",J,"K",K,if(is.na(L)) "" else paste0("_",L,"lags")));

# Bootstrapping network objects
nsim.se = 1000;
crashNetworksSim  = vector("list",length=nrow(subsetTimes));
for(s in 1:nrow(subsetTimes)){
  VECM = vecm[[s]];
  
  # Simulating VECM parameters
  # Generating parameter draws consistent with the mean and VCV of the estimated parameters
  # The simulation is inspired by http://www.quantumforest.com/2011/10/simulating-data-following-a-given-covariance-structure/
  coefCor = cov2cor(VECM$coefCov);
  simPar = t(t(chol(coefCor)) %*% matrix(rnorm(nrow(coefCor)*nsim.se), nrow=nrow(coefCor), ncol=nsim.se)); # Generating the correlation structure
  simPar = t((t(simPar)-colMeans(simPar))/apply(simPar,2,sd)); # Making sure that each column is standard normal
  VECM$simCoef = t(t(simPar)*sqrt(diag(VECM$coefCov))+as.vector(VECM$coeff)); # Imposing the mean and sd of the estimated parameters

  # Obtaining network objects for each VECM parameter draw
  tmp2 = vector("list",length=nsim.se);
  for(i in 1:nsim.se){
    tmp = simCIRnetwork(VECM=VECM,J=J, K=K, coeffVector = i, nlab = nlab);
    dimnames(tmp$PI)[[2]] = paste("PI",dimnames(tmp$PI)[[2]],sep=".");
    tmp2[[i]] = data.table(coeffSim  = i,
                           eventTime = 0:K,
                           tmp$pcorr,
                           tmp$corr,
                           tmp$PI,
                           conv = sum(tmp$conv>0.05) == 0)
  }
  crashNetworksSim[[s]] = do.call(rbind,tmp2);
  vecm[[s]] = VECM;
  rm(tmp,tmp2,VECM,simPar,coefCor);
  print(dimnames(subsetTimes)[[1]][s]);
  
  # Exporting bootstrap results
  save(list=c("vecm","crashNetworksSim","J","K","subsetTimes","nlab","N"),file="output/crashNetworksSim");
}
#load(file="output/crashNetworksSim");
