require(systemfit); # Used to estimate the VECM
require(vars);      # Used for lag selection information criteria
require(corpcor);   # Used to obtain partial correlations

#############################################
########## SIMULATION FUNCTIONS #############
#############################################

# Function for creating a data matrix of a chosen time span and data frequency
VECM.freq = function(firstSec, lastSec, dataFreqMs, markets= 1:N, dateCurrent = NA, priceLabel = "mid", timeLabel = "time", dateLabel = "date",...){
  if(!is.na(dateCurrent) & (dateCurrent < 19000000 | !is.numeric(dateCurrent))) stop("Wrong date format"); 

  # Checking what markets have data
  for(i in markets[length(markets):1]){
    if(length(quotes[[i]])==0){
      markets = markets[markets != i];
      print(paste0("There is no data for ",nlab[i]))
    } else if(length(quotes[[i]][(if(!is.na(dateCurrent)) quotes[[i]][,dateLabel]==dateCurrent else TRUE) & quotes[[i]][,timeLabel]>=firstSec & quotes[[i]][,timeLabel]<lastSec & !is.na(quotes[[i]][,priceLabel]),])<10){
      markets = markets[markets != i];
      print(paste0("There is no data for ",nlab[i]))
    } else if(is.na(dateCurrent) & length(unique(quotes[[i]][,dateLabel]))>1) stop(paste0("The data for ",nlab[i]," span several days"));
  }
  if(length(markets)<2){
    warning("Less than two markets have data for the given period");
    return(matrix(NA,0,1))
  } 
  
  times = (1000*firstSec/dataFreqMs):(1000*lastSec/dataFreqMs);
  qq = quotes[[markets[1]]][!is.na(quotes[[markets[1]]][,priceLabel]) & quotes[[markets[1]]][,timeLabel]>=firstSec & 
                              quotes[[markets[1]]][,timeLabel]<lastSec & if(!is.na(dateCurrent)) quotes[[markets[1]]][,dateLabel]==dateCurrent else TRUE,c(timeLabel,priceLabel)];
  dimnames(qq)[[2]][1] = "time";
  qq[,"time"] = trunc(qq[,"time"]*1000/dataFreqMs,0);
  qq = qq[c(diff(qq[,"time"])>0,TRUE),];
  QP = as.matrix(merge(cbind(times,NA),qq,by=1,all.x=TRUE))[,c(1,3)];
  dimnames(QP)[[2]][2] = nlab[markets[1]];
  for(i in markets[2:length(markets)]){
    qq = quotes[[i]][!is.na(quotes[[i]][,priceLabel]) & quotes[[i]][,timeLabel]>=firstSec & quotes[[i]][,timeLabel]<lastSec &
                       if(!is.na(dateCurrent)) quotes[[i]][,dateLabel]==dateCurrent else TRUE,c(timeLabel,priceLabel)];
    dimnames(qq)[[2]][1] = "time";
    qq[,"time"] = trunc(qq[,"time"]*1000/dataFreqMs,0);
    qq = qq[c(diff(qq[,"time"])>0,TRUE),];
    QP = merge(QP,qq,by=1,all.x=TRUE);
    dimnames(QP)[[2]][ncol(QP)] = nlab[i];
  }
  QP = na.locf(QP,na.rm=FALSE);
  QP = log(QP[,2:ncol(QP)]);
  return(QP);
}

# Wrapper function for estimating the VECM and reporting network objects
estimateVECM = function(fileName="VECMoutput", lagCriterion="SC(n)",...){
  # Available lag criteria: AIC(n)  HQ(n)  SC(n) FPE(n)
  
  # This function sets up and estimates the system of equations (using OLS from the function systemfit)
  estimateVECM.wrapped = function(p,L=NA,...){
    if(!is.matrix(p)) p=as.matrix(p);
    nlab = dimnames(p)[[2]];
    N = ncol(p);
    
    # Lag selection using an information criterion (default is SIC)
    if(is.na(L)){
      lagSelection = suppressWarnings(VARselect(p[rowSums(is.na(p))==0,],lag.max=20,type="both")$selection);
      L = max(lagSelection[lagCriterion]-1,1);
    }
    
    # Obtaining error correction variables (ec), and returns at chosen lags (d)
    ec = lag(zoo(p[,1]-p[,2:N]),-1);
    if(!is.matrix(ec)) ec = as.matrix(ec,length(ec),1);
    b = colMeans(ec,na.rm=TRUE);
    ec = t(t(ec) - b); 
    d = data.frame(cbind(diff(p),lag(zoo(diff(p)), -(1:L)),ec));
    dimnames(d)[[2]] = c(paste0(nlab,"_R"),paste0(rep(nlab,L),"_L",rep(1:L,each=N)),paste0(nlab[2:N],"_EC"));
    
    # Checking that there are enough observations to estimate the system
    if(sum(rowSums(is.na(d))==0) < (5*(ncol(d)-N))){
      print(paste0("There are ",(ncol(d)-N)," regressors but only ", sum(rowSums(is.na(d))==0)," non-missing observations. Returning NA."))
      return(NA)
    }
    if(sum(apply(d[rowSums(is.na(d))==0,],2,sd)==0)>0){
      print(paste("The variable(s)",colnames(d)[apply(d[rowSums(is.na(d))==0,],2,sd)==0],"have zero variation. Returning NA.",sep=" "))
      return(NA)
    }
    
    # Setting up the system of equations
    VECMmodel = vector("list", length=N);
    for(n in 1:N){
      VECMmodel[[n]] = formula(paste(dimnames(d)[[2]][n]," ~ ",paste(dimnames(d)[[2]][(N+1):ncol(d)],collapse=" + ")," - 1",sep=""));
    }
    
    # Estimating the system using systemfit (OLS)
    # Reporting the coefficient vector in matrix format (VECMcoeff)
    estVECM <- systemfit(VECMmodel, data = d)
    #summary(estVECM)
    VECMcoeff  = estVECM$eq[[1]]$coeff;
    for(n in 2:N) VECMcoeff = cbind(VECMcoeff,estVECM$eq[[n]]$coeff);
    dimnames(VECMcoeff)[[2]] = nlab;
    
    # Obtaining the model residuals in matrix format (res)
    res = matrix(NA,nrow(p)-1,N);
    for(n in 1:N){
      res[,n] = estVECM$eq[[n]]$residuals;
    }

    # Exporting the results
    return(x=list(nlab=nlab,
                  N=N,
                  L=L,
                  lagIC=lagSelection,
                  coefCov=estVECM$coefCov,
                  coeff=VECMcoeff,
                  mu=b,
                  residCov=estVECM$residCov,
                  residMean=colMeans(res,na.rm=T)));
    
    # nlab = names of markets in system
    # N = number of markets in system
    # L = number of lags
    # lagIC = lag selection criterion used
    # coefCov = VECM coefficient covariance matrix
    # mu = mean error correction 
    # residCov = residual covariance matrix
    # residMean = mean residuals
  }  
  
  # Using the VECM estimates to obtain the network objects
  CIRnetwork = function(VECM=VECM, J=J, K=K, nlab=nlab){
    # Impulse response function
    dP = vector("list",VECM$N); # price change l periods after the shock 
    cP = vector("list",VECM$N); # cumulative price change l periods after the shock
    psiInf = matrix(NA,VECM$N,VECM$N); # share of shock incorporated in prices, denoted psi in Eq. (4)
    psi = vector("list",K+1); # VMA coefficient matrix, one for each iteration in time

    for(n in 1:VECM$N){ # Looping across markets
      dP[[n]] = matrix(0,ncol=VECM$N,nrow=VECM$L+J,dimnames=list(-(VECM$L-1):J,VECM$nlab));
      cP[[n]] = matrix(0,ncol=VECM$N,nrow=VECM$L+J,dimnames=list(-(VECM$L-1):J,VECM$nlab));
      dP[[n]][VECM$L,n] = 1; # unit shock to market n
      cP[[n]][VECM$L,n] = 1; # unit shock to market n
      
      for(j in 1:J){ # IRF iterations forward in time
        dP[[n]][VECM$L+j,] <- matrix(t(dP[[n]][VECM$L+j-1:VECM$L,]),1,VECM$L*VECM$N) %*% VECM$coeff[1:(VECM$L*VECM$N),] + matrix(cP[[n]][VECM$L+j-1,1]-cP[[n]][VECM$L+j-1,2:VECM$N],1,VECM$N-1) %*% VECM$coeff[VECM$L*VECM$N+1:(VECM$N-1),];
        cP[[n]][VECM$L+j,] <- colSums(dP[[n]][1:(VECM$L+j),]); # Updating the cumulative prices
      }
      for(k in 0:K){ # Obtaining the VMA coefficient matrix for each step ahead, \tilde{Psi}_\tau
        if(n == 1) psi[[k+1]] = matrix(NA,VECM$N,VECM$N,dimnames=list(VECM$nlab,VECM$nlab));
        psi[[k+1]][,n] = cP[[n]][VECM$L+k,];
      }
      psiInf[,n] = cP[[n]][VECM$L+J,]; # Denoted psi in manuscript
    }
    # Convergence of CIRF (according to Eq. (4), conv as defined below should be zero)
    conv = apply(psiInf,2,max) - apply(psiInf,2,min);
    
    # Breaking if conv is NA -- happens when psiInf is exploding
    if(sum(is.na(conv))>0){
      print("The cumulative impulse response function does not converge. Returning NA.");
      return(conv = NA);
    }
    
    # Information shares upper and lower bounds, see Eq. (11) and (12)
    IS = matrix(ncol=2*VECM$N,nrow=VECM$N,dimnames=list(NULL,c(paste0("order",1:VECM$N),VECM$nlab)));
    for(n in 1:VECM$N){
      n.order = c(n:VECM$N,0:(n-1))[c(n:VECM$N,0:(n-1))>0];
      omega   = VECM$residCov[n.order,n.order];
      IS[n,]  = c(n.order,(as.vector((psiInf[1,n.order] %*% t(chol(omega)))^2) / as.vector(psiInf[1,n.order] %*% omega %*% psiInf[1,n.order]))[sort(n.order,i=T)$ix] );
    }
    
    # Partial correlation matrix (as well as standard correlation matrix)
    pc = matrix(nrow=K+1,ncol=length(nlab)^2,dimnames=list(0:K,paste0("pc",1:(length(nlab)^2)))); # Partial correlation matrix, one for each iteration in time
    sc = matrix(nrow=K+1,ncol=length(nlab)^2,dimnames=list(0:K,paste0("sc",1:(length(nlab)^2)))); # Correlation matrix, one for each iteration in time
    for(k in 0:K){
      # Partial correlations
      IRFcov = psi[[k+1]] %*% VECM$residCov %*% t(psi[[k+1]]);
      pcAll = matrix(NA,length(nlab),length(nlab),dimnames=list(nlab,nlab));
      pcAll[VECM$nlab,VECM$nlab] = cor2pcor(IRFcov);
      pc[k+1,] = as.vector(pcAll);

      # The following yields the same outcome, following Eq. (8) and (9) in the manuscript
      #invIRFcov = solve(IRFcov);
      #D = matrix(0,N,N);
      #diag(D) = diag(invIRFcov);
      #pcorr = - solve(sqrt(D)) %*% invIRFcov %*% solve(sqrt(D));
      #diag(pcorr) = -diag(pcorr);
      #dimnames(pcorr) = list(VECM$nlab,VECM$nlab)
      #round(matrix(pc[k+1,],ncol = VECM$N, nrow =VECM$N, dimnames=list(VECM$nlab,VECM$nlab)),3)
      #round(pcorr,3)
      
      # Standard correlations
      scAll = matrix(NA,length(nlab),length(nlab),dimnames=list(nlab,nlab));
      scAll[VECM$nlab,VECM$nlab] = cov2cor(IRFcov);
      sc[k+1,] = as.vector(scAll);
    }
    
    # Price Inefficiency (following Eq.(6) and (7) in the manuscript)
    PI = matrix(nrow=K+1,ncol=VECM$N,dimnames=list(0:K,VECM$nlab));
    for(k in 0:K){
      PI[k+1,] = abs(1 - as.vector(psi[[k+1]] %*% VECM$residCov %*% psiInf[1,]) / as.vector(psiInf[1,] %*% VECM$residCov %*% psiInf[1,]));
    }
    
    # Exporting the results  
    return(list(ISl   = apply(IS[,VECM$N+1:VECM$N],2,min), 
                ISu   = apply(IS[,VECM$N+1:VECM$N],2,max), 
                conv  = conv, 
                psi   = colMeans(psiInf), 
                PI    = PI, 
                pcorr = pc, 
                corr  = sc, 
                K     = K,  
                J     = J));
    # ISl = Information share lower bound
    # ISu = Information share upper bound
    # conv = Vector used to determine whether the IRF converges to the same value for all markets
    # psi = Long-term impact of shocks in each market
    # PI = Price inefficiency
    # pcorr = Partial correlations
    # corr = Standard correlations
    # K & J are the same as the function inputs
  }
  
  VECM = estimateVECM.wrapped(...)
  save(list="VECM", file=fileName)
  if(length(VECM)==1) return(NA) else {
    CIR = CIRnetwork(VECM=VECM, J=J, K=K, nlab=nlab)
    return(c(VECM,CIR));
  } 
}

# This function is the same as CIRnetwork above, except that it is designed for simulated VECM parameters
simCIRnetwork = function(VECM=VECM, J=J, K=K, nlab = nlab, coeffVector){
  N = length(nlab);
  
  # Impulse response function
  dP = vector("list",VECM$N); # price change l periods after the shock 
  cP = vector("list",VECM$N); # cumulative price change l periods after the shock
  psiInf = matrix(NA,VECM$N,VECM$N); # share of shock incorporated in prices, denoted psi in Eq. (4)
  psi = vector("list",K+1); # VMA coefficient matrix, one for each iteration in time
  
  # Specify which coefficient vector to use
  coeff = matrix(VECM$simCoef[coeffVector,],dim(VECM$coeff),dimnames=dimnames(VECM$coeff));
  
  for(n in 1:VECM$N){ # Looping across markets
    dP[[n]] = matrix(0,ncol=VECM$N,nrow=VECM$L+J,dimnames=list(-(VECM$L-1):J,VECM$nlab));
    cP[[n]] = matrix(0,ncol=VECM$N,nrow=VECM$L+J,dimnames=list(-(VECM$L-1):J,VECM$nlab));
    dP[[n]][VECM$L,n] = 1; # unit shock to market n
    cP[[n]][VECM$L,n] = 1; # unit shock to market n
    
    for(j in 1:J){ # IRF iterations forward in time
      dP[[n]][VECM$L+j,] <- matrix(t(dP[[n]][VECM$L+j-1:VECM$L,]),1,VECM$L*VECM$N) %*% coeff[1:(VECM$L*VECM$N),] + matrix(cP[[n]][VECM$L+j-1,1]-cP[[n]][VECM$L+j-1,2:VECM$N],1,VECM$N-1) %*% coeff[VECM$L*VECM$N+1:(VECM$N-1),];
      cP[[n]][VECM$L+j,] <- colSums(dP[[n]][1:(VECM$L+j),]); # Updating the cumulative prices
    }
    for(k in 0:K){ # Obtaining the VMA coefficient matrix for each step ahead, \tilde{Psi}_\tau
      if(n == 1) psi[[k+1]] = matrix(NA,VECM$N,VECM$N,dimnames=list(VECM$nlab,VECM$nlab));
      psi[[k+1]][,n] = cP[[n]][VECM$L+k,];
    }
    psiInf[,n] = cP[[n]][VECM$L+J,];
  }
  # Convergence of CIRF (according to Eq. (4), conv as defined below should be zero)
  conv = apply(psiInf,2,max) - apply(psiInf,2,min);
  
  # Breaking if conv is NA -- happens when psiInf is exploding
  if(sum(is.na(conv))>0){
    print("The cumulative impulse response function does not converge. Returning NA.");
    return(conv = NA);
  }
  
  # Information shares upper and lower bounds, see Eq. (11) and (12)
  IS = matrix(ncol=2*VECM$N,nrow=VECM$N,dimnames=list(NULL,c(paste0("order",1:VECM$N),VECM$nlab)));
  for(n in 1:VECM$N){
    n.order = c(n:VECM$N,0:(n-1))[c(n:VECM$N,0:(n-1))>0];
    omega   = VECM$residCov[n.order,n.order];
    IS[n,]  = c(n.order,(as.vector((psiInf[1,n.order] %*% t(chol(omega)))^2) / as.vector(psiInf[1,n.order] %*% omega %*% psiInf[1,n.order]))[sort(n.order,i=T)$ix] );
  }
  
  # Partial correlation matrix (as well as standard correlation matrix)
  pc = matrix(nrow=K+1,ncol=N^2,dimnames=list(0:K,paste0("pc",1:(N^2)))); # Partial correlation matrix, one for each iteration in time
  sc = matrix(nrow=K+1,ncol=N^2,dimnames=list(0:K,paste0("sc",1:(N^2)))); # Correlation matrix, one for each iteration in time
  for(k in 0:K){
    IRFcov = psi[[k+1]] %*% VECM$residCov %*% t(psi[[k+1]]);
    if(sum(is.na(IRFcov))==0){
      # Partial correlations
      pcAll = matrix(NA,N,N,dimnames=list(nlab,nlab));
      pcAll[VECM$nlab,VECM$nlab] = cor2pcor(IRFcov);
      pc[k+1,] = as.vector(pcAll);

      # standard correlations
      scAll = matrix(NA,N,N,dimnames=list(nlab,nlab));
      scAll[VECM$nlab,VECM$nlab] = cov2cor(IRFcov);
      sc[k+1,] = as.vector(scAll);
    }
  }
  
  # Price Inefficiency (following Eq.(6) and (7) in the manuscript)
  PI = matrix(nrow=K+1,ncol=VECM$N,dimnames=list(0:K,VECM$nlab)); # Cumulative impulse response matrix, one for each iteration in time
  for(k in 0:K){
    PI[k+1,] = abs(1 - as.vector(psi[[k+1]] %*% VECM$residCov %*% psiInf[1,]) / as.vector(psiInf[1,] %*% VECM$residCov %*% psiInf[1,]));
  }
  
  # Output  
  return(list(ISl=apply(IS[,VECM$N+1:VECM$N],2,min),
              ISu=apply(IS[,VECM$N+1:VECM$N],2,max),
              conv=conv,
              psi=colMeans(psiInf),
              PI = PI,
              pcorr = pc,
              corr = sc));
  # ISl = Information share lower bound
  # ISu = Information share upper bound
  # conv = Vector used to determine whether the IRF converges to the same value for all markets
  # psi = Long-term impact of shocks in each market
  # PI = Price inefficiency
  # pcorr = Partial correlations
  # corr = Standard correlations
}
